@@ -696,18 +696,6 @@ QString AbstractMetaFunction::targetLangSignature(bool minimal) const
 
     // Attributes...
     if (!minimal) {
-#if 0 // jambi
-        if (isPublic()) s += "public ";
-        else if (isProtected()) s += "protected ";
-        else if (isPrivate()) s += "private ";
-
-//     if (isNative()) s += "native ";
-//     else
-        if (isFinalInTargetLang()) s += "final ";
-        else if (isAbstract()) s += "abstract ";
-
-        if (isStatic()) s += "static ";
-#endif
         // Return type
         if (type())
             s += type()->name() + " ";
@@ -850,14 +838,24 @@ AbstractMetaFunctionList AbstractMetaClass::queryFunctionsByName(const QString &
     return returned;
 }
 
-bool AbstractMetaClass::hasDefaultIsNull() const 
+QString AbstractMetaClass::getDefaultNonZeroFunction() const
 {
-  foreach(const AbstractMetaFunction* fun, queryFunctionsByName("isNull")) {
-    if (fun->actualMinimumArgumentCount()==0) {
-      return true;
+  foreach(const AbstractMetaFunction* fun, queryFunctionsByName("isEmpty")) {
+    if (fun->actualMinimumArgumentCount()==0 && fun->isPublic()) {
+      return "isEmpty";
     }
   }
-  return false;
+  foreach(const AbstractMetaFunction* fun, queryFunctionsByName("isValid")) {
+    if (fun->actualMinimumArgumentCount() == 0 && fun->isPublic()) {
+      return "isValid";
+    }
+  }
+  foreach(const AbstractMetaFunction* fun, queryFunctionsByName("isNull")) {
+    if (fun->actualMinimumArgumentCount() == 0 && fun->isPublic()) {
+      return "isNull";
+    }
+  }
+  return QString();
 }
 
 /*******************************************************************************
