@@ -39,7 +39,16 @@
 **
 ****************************************************************************/
 
+// We need to force the endianess in Qt5
+#define Q_BYTE_ORDER Q_LITTLE_ENDIAN
+
 #define QT_NO_STL
+#define override 
+#define final
+#define QOPENGLVERSIONFUNCTIONS_H
+#define QOPENGLFUNCTIONS_H
+#define QOPENGLEXTRAFUNCTIONS_H
+
 #include <QtCore/QtCore>
 #include <QtGui/QtGui>
 #include <QtNetwork/QtNetwork>
@@ -51,12 +60,16 @@
 
 #if QT_VERSION >= 0x050000
 #include <QtWidgets/QtWidgets>
+#include <QtPrintSupport/QtPrintSupport>
 #endif
 
 #include <QtMultimedia/QtMultimedia>
 
 #if QT_VERSION >= 0x050000
 #include <QtMultimediaWidgets/QtMultimediaWidgets>
+#include <QtQml/QtQml>
+#include <QtQuick/QtQuick>
+#include <QtQuickWidgets/QtQuickWidgets>
 #endif
 
 #ifndef QT_NO_XMLPATTERNS
