@@ -46,6 +46,7 @@
 #include "PythonQtSignal.h"
 #include "PythonQtSignalReceiver.h"
 #include "PythonQtConversion.h"
+#include "PythonQtProperty.h"
 #include "PythonQtStdIn.h"
 #include "PythonQtStdOut.h"
 #include "PythonQtCppWrapperFactory.h"
@@ -53,6 +54,7 @@
 #include "PythonQtStdDecorators.h"
 #include "PythonQtQFileImporter.h"
 #include "PythonQtBoolResult.h"
+#include "PythonQtSlotDecorator.h"
 
 #include <QDir>
 
@@ -101,6 +103,27 @@ void PythonQt::init(int flags, const QByteArray& pythonQtModuleName)
     PythonQtRegisterToolClassesTemplateConverter(qint64);
     PythonQtRegisterToolClassesTemplateConverter(quint64);
     
+#ifdef PYTHONQT_SUPPORT_ML_TYPES
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLfloat>", "QList<float>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLfloat>", "QVector<float>");
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLdouble>", "QList<double>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLdouble>", "QVector<double>");
+
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLuint32>", "QList<quint32>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLuint32>", "QVector<quint32>");
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLint32>", "QList<qint32>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLint32>", "QVector<qint32>");
+
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLuint64>", "QList<quint64>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLuint64>", "QVector<quint64>");
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLint64>", "QList<qint64>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLint64>", "QVector<qint64>");
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLuint>", "QList<quint64>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLuint>", "QVector<quint64>");
+    PythonQtMethodInfo::addParameterTypeAlias("QList<MLint>", "QList<qint64>");
+    PythonQtMethodInfo::addParameterTypeAlias("QVector<MLint>", "QVector<qint64>");
+#endif
+
     PythonQtMethodInfo::addParameterTypeAlias("QList<qreal>", "QList<double>");
     PythonQtMethodInfo::addParameterTypeAlias("QVector<qreal>", "QVector<double>");
     PythonQtMethodInfo::addParameterTypeAlias("QList<unsigned int>", "QList<quint32>");
@@ -240,6 +263,11 @@ void PythonQt::init(int flags, const QByteArray& pythonQtModuleName)
     }
 
     _self->priv()->pythonQtModule().addObject("Debug", _self->priv()->_debugAPI);
+
+    PyModule_AddObject(pack, "Slot", (PyObject*)&PythonQtSlotDecorator_Type);
+    PyModule_AddObject(pack, "Signal", (PyObject*)&PythonQtSignalFunction_Type);
+    PyModule_AddObject(pack, "Property", (PyObject*)&PythonQtProperty_Type);
+
   }
 }
 
@@ -282,6 +310,16 @@ PythonQt::PythonQt(int flags, const QByteArray& pythonQtModuleName)
   }
   Py_INCREF(&PythonQtSignalFunction_Type);
 
+  if (PyType_Ready(&PythonQtSlotDecorator_Type) < 0) {
+    std::cerr << "could not initialize PythonQtSlotDecorator_Type" << ", in " << __FILE__ << ":" << __LINE__ << std::endl;
+  }
+  Py_INCREF(&PythonQtSlotDecorator_Type);
+
+  if (PyType_Ready(&PythonQtProperty_Type) < 0) {
+    std::cerr << "could not initialize PythonQtProperty_Type" << ", in " << __FILE__ << ":" << __LINE__ << std::endl;
+  }
+  Py_INCREF(&PythonQtProperty_Type);
+
   PythonQtBoolResult_Type.tp_new = PyType_GenericNew;
   if (PyType_Ready(&PythonQtBoolResult_Type) < 0) {
     std::cerr << "could not initialize PythonQtBoolResult_Type" << ", in " << __FILE__ << ":" << __LINE__ << std::endl;
@@ -519,9 +557,9 @@ PyObject* PythonQtPrivate::wrapPtr(void* ptr, const QByteArray& name, bool passO
     if (!info) {
       // maybe it is a PyObject, which we can return directly
       if (name == "PyObject") {
-        PyObject* p = (PyObject*)ptr;
-        Py_INCREF(p);
-        return p;
+        // do not increment its ref-count, it is the job of the slot returning the value
+        // to ensure an extra ref on return.
+        return (PyObject*)ptr;
       }
 
       // we do not know the metaobject yet, but we might know it by its name:
@@ -596,7 +634,7 @@ PyObject* PythonQtPrivate::wrapPtr(void* ptr, const QByteArray& name, bool passO
       info->setMetaObject(wrapper->metaObject());
     }
 
-    if (possibleStillAliveWrapper && possibleStillAliveWrapper->classInfo() == info) {
+    if (possibleStillAliveWrapper && possibleStillAliveWrapper->classInfo()->inherits(info)) {
       wrap = possibleStillAliveWrapper;
       Py_INCREF(wrap);
     } else {
@@ -877,12 +915,10 @@ QVariant PythonQt::evalScript(PyObject* object, const QString& script, int start
 
 void PythonQt::evalFile(PyObject* module, const QString& filename)
 {
+  // NOTE: error checking is done by parseFile and evalCode
   PythonQtObjectPtr code = parseFile(filename);
-  clearError();
   if (code) {
     evalCode(module, code);
-  } else {
-    handleError();
   }
 }
 
@@ -893,6 +929,7 @@ PythonQtObjectPtr PythonQt::parseFile(const QString& filename)
   clearError();
   if (!p) {
     handleError();
+    _p->_hadError = true;
   }
   return p;
 }
@@ -1388,6 +1425,18 @@ void PythonQtPrivate::removeSignalEmitter(QObject* obj)
   _signalReceivers.remove(obj);
 }
 
+void PythonQt::removeSignalHandlers()
+{
+  QList<PythonQtSignalReceiver*> signalReceivers = _p->_signalReceivers.values();
+
+  // just delete all signal receivers, they will remove themselves via removeSignalEmitter()
+  foreach(PythonQtSignalReceiver* receiver, signalReceivers) {
+    delete receiver;
+  }
+  // just to be sure, clear the receiver map as well
+  _p->_signalReceivers.clear();
+}
+
 namespace
 {
 //! adapted from python source file "pythonrun.c", function "handle_system_exit"
@@ -1879,6 +1928,196 @@ bool PythonQtPrivate::isMethodDescriptor(PyObject* object) const
   return false;
 }
 
+// We need this for the dynamic meta object building:
+#include <private/qmetaobjectbuilder_p.h>
+
+const QMetaObject* PythonQtPrivate::getDynamicMetaObject(PythonQtInstanceWrapper* wrapper, const QMetaObject* prototypeMetaObject)
+{
+  PythonQtDynamicClassInfo* info = wrapper->dynamicClassInfo();
+  if (info) {
+    if (!info->_dynamicMetaObject) {
+      setupDynamicMetaObjectChain(((PythonQtClassWrapper*)Py_TYPE(wrapper)), prototypeMetaObject);
+    }
+    return info->_dynamicMetaObject;
+  }
+  return prototypeMetaObject;
+}
+
+const QMetaObject* PythonQtPrivate::setupDynamicMetaObjectChain(PythonQtClassWrapper* type, const QMetaObject* prototypeMetaObject)
+{
+  if (!type->_dynamicClassInfo->_dynamicMetaObject) {
+    PyTypeObject*  superType = ((PyTypeObject *)type)->tp_base;
+    const QMetaObject* metaObjectOfParent = prototypeMetaObject;
+    if (((PythonQtClassWrapper*)superType)->_dynamicClassInfo) {
+      metaObjectOfParent = setupDynamicMetaObjectChain((PythonQtClassWrapper*)superType, prototypeMetaObject);
+    }
+    return buildDynamicMetaObject(type, metaObjectOfParent);
+  } else {
+    return type->_dynamicClassInfo->_dynamicMetaObject;
+  }
+}
+
+const QMetaObject* PythonQtPrivate::buildDynamicMetaObject(PythonQtClassWrapper* type, const QMetaObject* prototypeMetaObject)
+{
+  //std::cout << "creating " << ((PyTypeObject*)type)->tp_name << " derived from " << prototypeMetaObject->className() << std::endl;
+
+  QMetaObjectBuilder builder;
+  builder.setSuperClass(prototypeMetaObject);
+  builder.setClassName(((PyTypeObject*)type)->tp_name);
+
+  PyObject* dict = ((PyTypeObject*)type)->tp_dict;
+  Py_ssize_t pos = NULL;
+  PyObject* value = NULL;
+  PyObject* key = NULL;
+  static PyObject* qtSlots = PyString_FromString("_qtSlots");
+
+  bool needsMetaObject = false;
+  // Iterate over all members and check if they affect the QMetaObject:
+  // First look for signals:
+  while (PyDict_Next(dict, &pos, &key, &value)) {
+    if (PythonQtSignalFunction_Check(value)) {
+      // A signal object, register with the meta object
+      PythonQtSignalFunctionObject* signal = (PythonQtSignalFunctionObject*)value;
+      if (signal->_dynamicInfo) {
+        signal->_dynamicInfo->name = PyString_AsString(key);
+        foreach(QByteArray sig, signal->_dynamicInfo->signatures) {
+          QMetaMethodBuilder method = builder.addSignal(signal->_dynamicInfo->name + "(" + sig + ")");
+          needsMetaObject = true;
+        }
+      }
+    }
+  }
+  pos = NULL;
+  value = NULL;
+  key = NULL;
+  // Now look for slots: (this is a bug in QMetaObjectBuilder, all signals need to be added first)
+  while (PyDict_Next(dict, &pos, &key, &value)) {
+    if (PythonQtProperty_Check(value)) {
+      needsMetaObject = true;
+      PythonQtProperty* prop = (PythonQtProperty*)value;
+      QMetaPropertyBuilder newProp = builder.addProperty(PyString_AsString(key), prop->data->cppType);
+      newProp.setReadable(true);
+      newProp.setWritable(prop->data->fset != NULL);
+      newProp.setResettable(prop->data->freset != NULL);
+      newProp.setDesignable(prop->data->designable);
+      newProp.setScriptable(prop->data->scriptable);
+      newProp.setStored(prop->data->stored);
+      newProp.setUser(prop->data->user);
+      newProp.setConstant(prop->data->constant);
+      newProp.setFinal(prop->data->final);
+      if (prop->data->notify) {
+        PythonQtSignalFunctionObject* signal = (PythonQtSignalFunctionObject*)prop->data->notify;
+        if (signal->_dynamicInfo) {
+          QByteArray sig = signal->_dynamicInfo->signatures.at(0);
+          QByteArray fullSig = signal->_dynamicInfo->name + "(" + sig + ")";
+          int idx = builder.indexOfSignal(fullSig);
+          if (idx != -1) {
+            newProp.setNotifySignal(builder.method(idx));
+          } else {
+            std::cerr << "could not find notify signal signature " << fullSig.constData();
+          }
+        }
+      }
+    }
+    if (PyFunction_Check(value) && PyObject_HasAttr(value, qtSlots)) {
+      // A function which has a "_qtSlots" signature list, add the slots to the meta object
+      PyObject* signatures = PyObject_GetAttr(value, qtSlots);
+      Py_ssize_t count = PyList_Size(signatures);
+      for (Py_ssize_t i = 0; i < count; i++) {
+        PyObject* signature = PyList_GET_ITEM(signatures, i);
+        QByteArray sig = PyString_AsString(signature);
+        // Split the return type and the rest of the signature,
+        // no spaces should be in the rest of the signature...
+        QList<QByteArray> parts = sig.split(' ');
+        QMetaMethodBuilder method = builder.addSlot(parts[1]);
+        // set the return type of the slot
+        method.setReturnType(parts[0]);
+        needsMetaObject = true;
+      }
+    }
+    // TODO: handle enums, classinfo, ...
+  }
+  if (needsMetaObject) {
+    type->_dynamicClassInfo->_dynamicMetaObject = builder.toMetaObject();
+    type->_dynamicClassInfo->_classInfo = new PythonQtClassInfo();
+    type->_dynamicClassInfo->_classInfo->setupQObject(type->_dynamicClassInfo->_dynamicMetaObject);
+  } else {
+    // we don't need an own meta object, just use the one from our base class
+    type->_dynamicClassInfo->_dynamicMetaObject = prototypeMetaObject;
+  }
+  return type->_dynamicClassInfo->_dynamicMetaObject;
+}
+
+
+int PythonQtPrivate::handleMetaCall(QObject* object, PythonQtInstanceWrapper* wrapper, QMetaObject::Call call, int id, void** args)
+{
+  const QMetaObject* meta = object->metaObject();
+  int methodCount = meta->methodCount();
+  if (call == QMetaObject::InvokeMetaMethod) {
+    QMetaMethod method = meta->method(id);
+    if (method.methodType() == QMetaMethod::Signal) {
+      // just emit the signal, there is no Python code
+      QMetaObject::activate(object, id, args);
+    } else {
+      callMethodInPython(method, wrapper, args);
+    }
+  } else {
+    QMetaProperty metaProp = meta->property(id);
+    if (!metaProp.isValid()) {
+      return id - methodCount;
+    }
+    PythonQtProperty* prop = NULL;
+    // Get directly from the Python class, since we don't want to get the value of the property
+    PyObject* maybeProp = PyBaseObject_Type.tp_getattro((PyObject*)wrapper, PyString_FromString(metaProp.name()));
+    if (maybeProp && PythonQtProperty_Check(maybeProp)) {
+      prop = (PythonQtProperty*)maybeProp;
+    } else {
+      return id - methodCount;
+    }
+    const PythonQtMethodInfo::ParameterInfo& info = PythonQtMethodInfo::getParameterInfoForMetaType(metaProp.userType());
+
+    if (call == QMetaObject::WriteProperty) {
+      PyObject* value = PythonQtConv::ConvertQtValueToPython(info, args[0]);
+      bool ok = prop->data->callSetter((PyObject*)wrapper, value);
+      Py_XDECREF(value);
+
+      return ok ? 0 : -1;
+
+    } else if (call == QMetaObject::ReadProperty) {
+
+      PyObject* value = prop->data->callGetter((PyObject*)wrapper);
+      if (value) {
+        void* result = PythonQtConv::ConvertPythonToQt(info, value, false, NULL, args[0]);
+        Py_DECREF(value);
+        return (result == NULL ? -1 : 0);
+      } else {
+        return -1;
+      }
+    } else if (call == QMetaObject::ResetProperty) {
+      bool ok = prop->data->callReset((PyObject*)wrapper);
+      return ok ? 0 : -1;
+    }
+  }
+  return id - methodCount;
+}
+
+void PythonQtPrivate::callMethodInPython(QMetaMethod &method, PythonQtInstanceWrapper* wrapper, void** args)
+{
+  QByteArray methodSig = method.methodSignature();
+  PyObject* func = PyObject_GetAttrString((PyObject*)wrapper, method.name());
+  if (func) {
+    const PythonQtMethodInfo* methodInfo = PythonQtMethodInfo::getCachedMethodInfo(method, NULL);
+    PyObject* result = PythonQtSignalTarget::call(func, methodInfo, args, false);
+    if (result) {
+      PythonQtConv::ConvertPythonToQt(methodInfo->parameters().at(0), result, false, NULL, args[0]);
+      // TODO: handle error?
+    //PythonQt::priv()->handleVirtualOverloadReturnError("devType", methodInfo, result);
+    }
+    Py_XDECREF(result);
+    Py_DECREF(func);
+  }
+}
+
 QString PythonQtPrivate::getSignature(PyObject* object)
 {
   QString signature;
