@@ -61,6 +61,7 @@ PythonQtClassInfo::PythonQtClassInfo() {
   _typeSlots = 0;
   _isQObject = false;
   _enumsCreated = false;
+  _richCompareDetectionDone = false;
   _searchPolymorphicHandlerOnParent = true;
   _searchRefCountCB = true;
   _refCallback = NULL;
@@ -134,6 +135,7 @@ bool PythonQtClassInfo::lookForPropertyAndCache(const char* memberName)
   const char* attributeName = memberName;
   // look for properties
   int i = _meta->indexOfProperty(attributeName);
+#ifdef PYTHONQT_SUPPORT_NAME_PROPERTY
   if (i==-1) {
     // try to map name to objectName
     if (qstrcmp(attributeName, "name")==0) {
@@ -142,6 +144,13 @@ bool PythonQtClassInfo::lookForPropertyAndCache(const char* memberName)
       i = _meta->indexOfProperty(attributeName);
     }
   }
+#endif
+  if (qstrcmp(attributeName, "singleShot") == 0 &&
+      className() == "QTimer") {
+    // ignore singleShot property, users need to use setSingleShot and isSingleShot instead...
+    i = -1;
+  }
+
   if (i!=-1) {
     PythonQtMemberInfo newInfo(_meta->property(i));
     _cachedMembers.insert(attributeName, newInfo);
@@ -281,7 +290,7 @@ bool PythonQtClassInfo::lookForEnumAndCache(const QMetaObject* meta, const char*
           found = true;
           break;
         } else {
-          std::cout << "enum " << e.name() << " not found on " << className().constData() << std::endl;
+          std::cerr << "enum " << e.name() << " not found on " << className().constData() << std::endl;
         }
       }
     }
@@ -729,14 +738,14 @@ QObject* PythonQtClassInfo::decorator()
       _decoratorProvider->setParent(PythonQt::priv());
       // setup enums early, since they might be needed by the constructor decorators:
       if (!_enumsCreated) {
-        createEnumWrappers();
+        createEnumWrappers(_decoratorProvider);
       }
       PythonQt::priv()->addDecorators(_decoratorProvider, PythonQtPrivate::ConstructorDecorator | PythonQtPrivate::DestructorDecorator);
     }
   }
   // check if enums need to be created and create them if they are not yet created
   if (!_enumsCreated) {
-    createEnumWrappers();
+    createEnumWrappers(_decoratorProvider);
   }
   return _decoratorProvider;
 }
@@ -848,18 +857,20 @@ void PythonQtClassInfo::createEnumWrappers(const QMetaObject* meta)
   }
 }
 
-void PythonQtClassInfo::createEnumWrappers()
+void PythonQtClassInfo::createEnumWrappers(const QObject* decoratorProvider)
 {
   if (!_enumsCreated) {
     _enumsCreated = true;
     if (_meta) {
       createEnumWrappers(_meta);
     }
-    if (decorator()) {
-      createEnumWrappers(decorator()->metaObject());
+    if (decoratorProvider) {
+      createEnumWrappers(decoratorProvider->metaObject());
     }
     Q_FOREACH(const ParentClassInfo& info, _parentClasses) {
-      info._parent->createEnumWrappers();
+      // trigger decorator() instead of createEnumWrappers(),
+      // which will then call createEnumWrappers().
+      info._parent->decorator();
     }
   }
 }
@@ -867,7 +878,9 @@ void PythonQtClassInfo::createEnumWrappers()
 PyObject* PythonQtClassInfo::findEnumWrapper(const char* name) {
   // force enum creation
   if (!_enumsCreated) {
-    createEnumWrappers();
+    // trigger decorator() instead of createEnumWrappers(),
+    // which will then call createEnumWrappers().
+    decorator();
   }
   Q_FOREACH(const PythonQtObjectPtr& p, _enumWrappers) {
     const char* className = ((PyTypeObject*)p.object())->tp_name;
@@ -1021,11 +1034,42 @@ PythonQtClassInfo* PythonQtClassInfo::getClassInfoForProperty( const QString& na
     }
   }
   if (!typeName.isEmpty()) {
+    if (typeName.endsWith("*")) {
+      typeName.truncate(typeName.length() - 1);
+    }
     PythonQtClassInfo* classInfo = PythonQt::priv()->getClassInfo(typeName);
     return classInfo;
   }
   return NULL;
 }
+
+bool PythonQtClassInfo::supportsRichCompare()
+{
+  if (_typeSlots & PythonQt::Type_RichCompare) {
+    return true;
+  }
+  if (!_richCompareDetectionDone) {
+    _richCompareDetectionDone = true;
+    static QList<QByteArray> names;
+    if (names.isEmpty()) {
+      names << "__eq__";
+      names << "__ne__";
+      names << "__lt__";
+      names << "__le__";
+      names << "__gt__";
+      names << "__ge__";
+    }
+    foreach (const QByteArray& name, names) {
+      if (member(name)._type == PythonQtMemberInfo::Slot) {
+        // we found one of the operators, so we can support the type slot
+        _typeSlots |= PythonQt::Type_RichCompare;
+        break;
+      }
+    }
+  }
+  return (_typeSlots & PythonQt::Type_RichCompare);
+}
+
 //-------------------------------------------------------------------------
 
 PythonQtMemberInfo::PythonQtMemberInfo( PythonQtSlotInfo* info )
@@ -1055,4 +1099,9 @@ PythonQtMemberInfo::PythonQtMemberInfo( const QMetaProperty& prop )
   _property = prop;
   _enumValue = NULL;
   _pythonType = NULL;
-}
\ No newline at end of file
+}
+
+PythonQtDynamicClassInfo::~PythonQtDynamicClassInfo()
+{
+  delete _classInfo;
+}
