@@ -150,7 +150,6 @@ static PyObject* PythonQtInstanceWrapper_binaryfunc(PyObject* self, PyObject* ot
   PyObject* result = NULL;
   PythonQtMemberInfo opSlot = wrapper->classInfo()->member(opName);
   if (opSlot._type == PythonQtMemberInfo::Slot) {
-    // TODO get rid of tuple
     PyObject* args = PyTuple_New(1);
     Py_INCREF(other);
     PyTuple_SET_ITEM(args, 0, other);
@@ -178,7 +177,6 @@ static PyObject* PythonQtInstanceWrapper_mul(PyObject* self, PyObject* other)
   PyObject* result = NULL;
   PythonQtMemberInfo opSlot = wrapper->classInfo()->member("__mul__");
   if (opSlot._type == PythonQtMemberInfo::Slot) {
-    // TODO get rid of tuple
     PyObject* args = PyTuple_New(1);
     Py_INCREF(other);
     PyTuple_SET_ITEM(args, 0, other);
@@ -345,6 +343,7 @@ static int PythonQtClassWrapper_init(PythonQtClassWrapper* self, PyObject* args,
   if (PyType_Type.tp_init((PyObject *)self, args, kwds) < 0) {
     return -1;
   }
+  self->_dynamicClassInfo = NULL;
 
   // if we have no CPP class information, try our base class
   if (!self->classInfo()) {
@@ -367,6 +366,17 @@ static int PythonQtClassWrapper_init(PythonQtClassWrapper* self, PyObject* args,
 
     // take the class info from the superType
     self->_classInfo = ((PythonQtClassWrapper*)superType)->classInfo();
+    self->_dynamicClassInfo = new PythonQtDynamicClassInfo();
+
+    // take the class info from the superType and fill the whole chain
+    PyTypeObject* typeChain = (PyTypeObject *)self;
+    while (typeChain && Py_TYPE(typeChain) != &PythonQtClassWrapper_Type) {
+
+      ((PythonQtClassWrapper*)typeChain)->_classInfo = ((PythonQtClassWrapper*)superType)->classInfo();
+      ((PythonQtClassWrapper*)typeChain)->_dynamicClassInfo = new PythonQtDynamicClassInfo();
+
+      typeChain = typeChain->tp_base;
+    }
   }
 
   return 0;
@@ -456,12 +466,10 @@ static PyObject *PythonQtClassWrapper_getattro(PyObject *obj, PyObject *name)
     }
     PyObject* dict = PyDict_New();
       
-    QStringList l = wrapper->classInfo()->memberList();
-    Q_FOREACH (QString name, l) {
-      if (name.startsWith("py_get_")) {
-        // add slot getters as normal properties
-        name = name.mid(7);
-      }
+    QSet<QString> completeSet = QSet<QString>::fromList(wrapper->classInfo()->memberList());
+    completeSet.unite(QSet<QString>::fromList(wrapper->classInfo()->propertyList()));
+
+    Q_FOREACH (QString name, completeSet) {
       if (name.startsWith("py_")) {
         // do not expose internal slots
         continue;
@@ -512,8 +520,19 @@ static PyObject *PythonQtClassWrapper_getattro(PyObject *obj, PyObject *name)
       Py_INCREF(enumWrapper);
       return enumWrapper;
     } else if (member._type == PythonQtMemberInfo::Slot) {
-      // we return all slots, even the instance slots, since they are callable as unbound slots with self argument
-      return PythonQtSlotFunction_New(member._slot, obj, NULL);
+      // NOTE: this does an extra lookup and string copy, it might be better to move this to
+      //       PythonQtClassInfo and add a new cache for qualified class methods...
+      QByteArray qualifiedMemberName = "py_q_";
+      qualifiedMemberName += attributeName;
+      PythonQtMemberInfo qualifiedMember = wrapper->classInfo()->member(qualifiedMemberName);
+      if (qualifiedMember._type == PythonQtMemberInfo::Slot) {
+        // return the qualified member, so that virtual calls on classes call the qualified member
+        return PythonQtSlotFunction_New(qualifiedMember._slot, obj, NULL);
+      }
+      else {
+        // we return all slots, even the instance slots, since they are callable as unbound slots with self argument
+        return PythonQtSlotFunction_New(member._slot, obj, NULL);
+      }
     } else if (member._type == PythonQtMemberInfo::Signal) {
       // we return all signals, even the instance signals, since they are callable as unbound signals with self argument
       return PythonQtSignalFunction_New(member._slot, obj, NULL);
