@@ -46,6 +46,7 @@
 #include <QTime>
 #include <QDate>
 #include <climits>
+#include <limits>
 
 PythonQtValueStorage<qint64, 128>  PythonQtConv::global_valueStorage;
 PythonQtValueStorage<void*, 128>   PythonQtConv::global_ptrStorage;
@@ -54,6 +55,8 @@ PythonQtValueStorageWithCleanup<QVariant, 128> PythonQtConv::global_variantStora
 QHash<int, PythonQtConvertMetaTypeToPythonCB*> PythonQtConv::_metaTypeToPythonConverters;
 QHash<int, PythonQtConvertPythonToMetaTypeCB*> PythonQtConv::_pythonToMetaTypeConverters;
 
+PythonQtConvertPythonSequenceToQVariantListCB* PythonQtConv::_pythonSequenceToQVariantListCB = NULL;
+
 PyObject* PythonQtConv::GetPyBool(bool val)
 {
   PyObject* r = val?Py_True:Py_False;
@@ -101,7 +104,7 @@ PyObject* PythonQtConv::ConvertQtValueToPython(const PythonQtMethodInfo::Paramet
     }
   }
 
-  if (info.typeId >= QMetaType::User) {
+  if (info.typeId >= QMetaType::User || info.typeId == QMetaType::QByteArrayList) {
     // if a converter is registered, we use is:
     PythonQtConvertMetaTypeToPythonCB* converter = _metaTypeToPythonConverters.value(info.typeId);
     if (converter) {
@@ -383,7 +386,9 @@ void* PythonQtConv::ConvertPythonToQt(const PythonQtMethodInfo::ParameterInfo& i
      return ptr;
    }
 
-   if (PyObject_TypeCheck(obj, &PythonQtInstanceWrapper_Type) && info.typeId != PythonQtMethodInfo::Variant) {
+   if (PyObject_TypeCheck(obj, &PythonQtInstanceWrapper_Type) &&
+       info.typeId != PythonQtMethodInfo::Variant &&
+       !PythonQt::priv()->isPythonQtObjectPtrMetaId(info.typeId)) {
      // if we have a Qt wrapper object and if we do not need a QVariant, we do the following:
      // (the Variant case is handled below in a switch)
 
@@ -459,7 +464,6 @@ void* PythonQtConv::ConvertPythonToQt(const PythonQtMethodInfo::ParameterInfo& i
            bool ok;
            int value = PyObjGetInt(obj, true, ok);
            if (ok && value==0) {
-             // TODOXXX is this wise? or should it be expected from the programmer to use None?
              PythonQtValueStorage_ADD_VALUE_IF_NEEDED(alreadyAllocatedCPPObject,global_ptrStorage, void*, NULL, ptr);
            }
          }
@@ -666,7 +670,7 @@ void* PythonQtConv::ConvertPythonToQt(const PythonQtMethodInfo::ParameterInfo& i
          }
 
          // We only do this for registered type > QMetaType::User for performance reasons.
-         if (info.typeId >= QMetaType::User) {
+         if (info.typeId >= QMetaType::User || info.typeId == QMetaType::QByteArrayList) {
            // Maybe we have a special converter that is registered for that type:
            PythonQtConvertPythonToMetaTypeCB* converter = _pythonToMetaTypeConverters.value(info.typeId);
            if (converter) {
@@ -744,9 +748,14 @@ QString PythonQtConv::PyObjGetRepresentation(PyObject* val)
 QString PythonQtConv::PyObjGetString(PyObject* val, bool strict, bool& ok) {
   QString r;
   ok = true;
+#ifndef PY3K
+  // in Python 3, we don't want to convert to QString, since we don't know anything about the encoding
+  // in Python 2, we assume the default for str is latin-1
   if (val->ob_type == &PyBytes_Type) {
-    r = QString(PyBytes_AS_STRING(val));
-  } else if (PyUnicode_Check(val)) {
+    r = QString::fromLatin1(PyBytes_AS_STRING(val));
+  } else
+#endif
+  if (PyUnicode_Check(val)) {
 #ifdef PY3K
     r = QString::fromUtf8(PyUnicode_AsUTF8(val));
 #else
@@ -974,8 +983,15 @@ QVariant PythonQtConv::PyObjToQVariant(PyObject* val, int type)
 #endif
     ) {
     // no special type requested
-    if (PyBytes_Check(val) || PyUnicode_Check(val)) {
-      // NOTE: for compatibility reasons between Python 2/3 we don't use ByteArray for PyBytes_Type
+    if (PyBytes_Check(val)) {
+#ifdef PY3K
+      // In Python 3, it is a ByteArray
+      type = QVariant::ByteArray;
+#else
+      // In Python 2, we need to use String, since it might be a string
+      type = QVariant::String;
+#endif
+    } else if (PyUnicode_Check(val)) {
       type = QVariant::String;
     } else if (val == Py_False || val == Py_True) {
       type = QVariant::Bool;
@@ -984,7 +1000,15 @@ QVariant PythonQtConv::PyObjToQVariant(PyObject* val, int type)
       type = QVariant::Int;
 #endif
     } else if (PyLong_Check(val)) {
-      type = QVariant::LongLong;
+      // return int if the value fits into that range,
+      // otherwise it would not be possible to get an int from Python 3
+      qint64 d = PyLong_AsLongLong(val);
+      if (d > std::numeric_limits<int>::max() ||
+          d < std::numeric_limits<int>::min()) {
+        type = QVariant::LongLong;
+      } else {
+        type = QVariant::Int;
+      }
     } else if (PyFloat_Check(val)) {
       type = QVariant::Double;
     } else if (PyObject_TypeCheck(val, &PythonQtInstanceWrapper_Type)) {
@@ -1006,13 +1030,13 @@ QVariant PythonQtConv::PyObjToQVariant(PyObject* val, int type)
         v = qVariantFromValue(myObject);
       }
       return v;
+    } else if (val == Py_None) {
+      // none is invalid
+      type = QVariant::Invalid;
     } else if (PyDict_Check(val)) {
       type = QVariant::Map;
     } else if (PyList_Check(val) || PyTuple_Check(val) || PySequence_Check(val)) {
       type = QVariant::List;
-    } else if (val == Py_None) {
-      // none is invalid
-      type = QVariant::Invalid;
     } else {
       // this used to be:
       // type = QVariant::String;
@@ -1107,6 +1131,14 @@ QVariant PythonQtConv::PyObjToQVariant(PyObject* val, int type)
     }
 
   case QVariant::ByteArray:
+    {
+      bool ok;
+#ifdef PY3K
+      v = QVariant(PyObjGetBytes(val, false, ok));
+#else
+      v = QVariant(PyObjGetString(val, false, ok));
+#endif
+    }
   case QVariant::String:
     {
       bool ok;
@@ -1121,20 +1153,30 @@ QVariant PythonQtConv::PyObjToQVariant(PyObject* val, int type)
     pythonToMapVariant<QVariantHash>(val, v);
     break;
   case QVariant::List:
-    if (PySequence_Check(val)) {
+  {
+    bool isListOrTuple = PyList_Check(val) || PyTuple_Check(val);
+    if (isListOrTuple || PySequence_Check(val)) {
+      if (!isListOrTuple && _pythonSequenceToQVariantListCB) {
+        // Only call this if we don't have a tuple or list.
+        QVariant result = (*_pythonSequenceToQVariantListCB)(val);
+        if (result.isValid()) {
+          return result;
+        }
+      }
       int count = PySequence_Size(val);
       if (count >= 0) {
         // only get items if size is valid (>= 0)
         QVariantList list;
         PyObject* value;
-        for (int i = 0;i<count;i++) {
-          value = PySequence_GetItem(val,i);
+        for (int i = 0; i < count; i++) {
+          value = PySequence_GetItem(val, i);
           list.append(PyObjToQVariant(value, -1));
           Py_XDECREF(value);
         }
         v = list;
       }
     }
+  }
     break;
   case QVariant::StringList:
     {
@@ -1450,6 +1492,56 @@ PyObject* PythonQtConv::convertFromStringRef(const void* inObject, int /*metaTyp
   return PythonQtConv::QStringToPyObject(((QStringRef*)inObject)->toString());
 }
 
+QByteArray PythonQtConv::getCPPTypeName(PyObject* type)
+{
+  QByteArray result;
+  if (PyType_Check(type)) {
+    if (type->ob_type == &PythonQtClassWrapper_Type) {
+      PythonQtClassWrapper* wrapper = (PythonQtClassWrapper*)type;
+      // we have no good decision if this should be a pointer or a const reference
+      if (wrapper->classInfo()->isQObject()) {
+        // for QObjects, use a pointer
+        result = wrapper->classInfo()->className() + "*";
+      } else {
+        // everything else is a const reference (otherwise the user needs to use strings to specify the type)
+        result = wrapper->classInfo()->className();
+      }
+    } else {
+      PyTypeObject* typeObject = reinterpret_cast<PyTypeObject*>(type);
+      if (typeObject == &PyFloat_Type) {
+        result = "double";
+      } else if (typeObject == &PyBool_Type) {
+        result = "bool";
+#ifndef PY3K
+      } else if (typeObject == &PyInt_Type) {
+        result = "qint32";
+#endif
+      } else if (typeObject == &PyLong_Type) {
+        result = "qint64";
+      } else if (isStringType(typeObject)) {
+        result = "QString";
+      } else {
+        result = "PyObject*";
+      }
+    }
+  } else if (type == Py_None) {
+      result = "void";
+  } else {
+    bool dummy;
+    result = QMetaObject::normalizedType(PyObjGetString(type, true, dummy).toLatin1());
+  }
+  return result;
+}
+
+bool PythonQtConv::isStringType(PyTypeObject* type)
+{
+#ifdef PY3K
+  return type == &PyUnicode_Type;
+#else
+  return type == &PyUnicode_Type || type == &PyString_Type;
+#endif
+}
+
 PyObject* PythonQtConv::convertFromQListOfPythonQtObjectPtr(const void* inObject, int /*metaTypeId*/)
 {
   QList<PythonQtObjectPtr>& list = *((QList<PythonQtObjectPtr>*)inObject);
