@@ -60,9 +60,8 @@ Qt framework</a>.
  your C++ Qt applications.
 
  The focus of PythonQt is on embedding Python into an existing C++ application, not on writing the whole
- application completely in Python. If you want to write your whole application in Python,
- you should use <a href="http://www.riverbankcomputing.co.uk/pyqt/" target="_blank">PyQt</a> or <a href="http://www.pyside.org" target="_blank">PySide</a> instead.
-
+ application completely in Python.
+ 
  If you are looking for a simple way to embed Python objects into your C++/Qt Application
  and to script parts of your application via Python, PythonQt is the way to go!
 
@@ -101,7 +100,8 @@ Qt framework</a>.
  - Deriving C++ objects from Python and overwriting virtual method with a Python implementation (requires usage of wrapper generator or manual work!)
  - Extensible handler for Python/C++ conversion of complex types, e.g. mapping of QVector<SomeObject> to/from a Python array
  - Setting of dynamic QObject properties via setProperty(), dynamic properties can be accessed for reading and writing like normal Python attributes (but creating a new property needs to be done with setProperty(), to distinguish from normal Python attributes)
-
+ - Support for QtCore.Signal, QtCore.Slot and QtCore.Property, including the creation of a dynamic QMetaObject.
+ 
  \section FeaturesQtAll Features with wrapper generator
 
  PythonQt offers the additional PythonQt_QtAll library which wraps the complete Qt API, including all C++ classes and all non-slots on QObject derived classes.
@@ -115,38 +115,37 @@ Qt framework</a>.
   - QtOpenGL
   - QtSql
   - QtSvg
-  - QtUiTools
   - QtWebKit
   - QtXml
   - QtXmlPatterns
-  - (QtMultiMedia, QtScript, QtHelp, phonon, assistant, designer are currently not supported, this would require some additional effort on the code generator)
- - For convenience, all classes are also available in the PythonQt.Qt module, for people who do not care in which module a class is located
+  - QtMultimedia
+  - QtQml
+  - QtQuick
  - Any Qt class that has virtual methods can be easily derived from Python and the virtual methods can be reimplemented in Python
  - Polymorphic downcasting on QEvent, QGraphicsItem, QStyleOption, ...
  - Multiple inheritance support (e.g., QGraphicsTextItem is a QObject and a QGraphicsItem, PythonQt will handle this well)
-
+ - QtQuick support is experimental and currently it is not possible to register new qml components from Python
+ 
  \section Supported Supported Versions
 
  PythonQt supports:
  - Python 2 (>= Python 2.6)
  - Python 3 (>= Python 3.3)
  - Qt 4.x (Qt 4.7 and Qt 4.8 recommended)
- - Qt 5.x (Tested with Qt 5.0 and 5.3)
+ - Qt 5.x (Tested with Qt 5.0, 5.3, 5.4 and 5.6)
  
- The Qt5 support does not yet include Qt modules that have been introduced with Qt5. The new QtWidgets module is wrapped as part of QtGui.
- This allows to run scripts that were written with the Qt4 binding on a Qt5 PythonQt without porting.
+ The last working Qt4 version is available at svn branches/Qt4LastWorkingVersion or you can download the PythonQt 3.0 release.
+ The current svn trunk no longer supports Qt4, since we started to make use of some Qt5-only features.
 
- \section Comparison Comparison with PyQt/PySide
+ \section Comparison Comparison with PySide
 
- - PythonQt is not as pythonic as PyQt/PySide in many details (e.g. buffer protocol, pickling, translation support, ...) and it is mainly thought for embedding and intercommunication between Qt/Cpp and Python
- - PythonQt allows to communicate in both directions, e.g., calling a Python object from C++ AND calling a C++ method from Python, while PyQt only handles the Python->C++ direction
- - PythonQt offers properties as Python attributes, while PyQt offers them as setter/getter methods (e.g. QWidget.width is a property in PythonQt and a method in PyQt)
+ - PythonQt is not as pythonic as PySide in many details (e.g. buffer protocol, pickling, translation support, ...) and it is mainly thought for embedding and intercommunication between Qt/Cpp and Python
+ - PythonQt offers properties as Python attributes, while PySide offers them as setter/getter methods (e.g. QWidget.width is a property in PythonQt and a method in PySide)
  - PythonQt currently does not support instanceof checks for Qt classes, except for the exact match and derived Python classes
  - QObject.emit to emit Qt signals from Python is not yet implemented but PythonQt allows to just emit a signal by calling it like a normal slot
- - PythonQt does not (yet) offer to add new signals/slots to Python/C++ objects and it does not create QMetaObjects for derived classes on the fly
- - Ownership handling of objects is not as complete as in PySide and PyQt, especially in situations where the ownership is not clearly passed to C++ on the C++ API.
+ - Ownership handling of objects is not as complete as in PySide and PySide, especially in situations where the ownership is not clearly passed to C++ on the C++ API.
  - QStrings are always converted to unicode Python objects, QByteArray always stays a QByteArray and can be converted using QByteArray.data()
- - Qt methods that take an extra "bool* ok" parameter can be called passing PythonQt.BoolResult as parameter. In PyQt, a tuple is returned instead.
+ - Qt methods that take an extra "bool* ok" parameter can be called passing PythonQt.BoolResult as parameter. In PySide, a tuple is returned instead.
  
  \page Download Download
 
@@ -162,8 +161,7 @@ Qt framework</a>.
 
  \page License License
   
- PythonQt is distributed under the LGPL license, so it pairs well with the LGPL of the Qt 4.5 release and allows
- to be used in commercial applications when following the LGPL 2.1 obligations.
+ PythonQt is distributed under the LGPL 2.1 license. It can be used in commercial applications when following the LGPL 2.1 obligations.
 
  The build system of PythonQt makes use of a modified version of the LGPL'ed QtScript generator,
  located in the "generator" directory.
@@ -174,7 +172,7 @@ Qt framework</a>.
 
  The PythonQt wrappers generated by the generator located in the "generated_cpp" directory are free to be used without any licensing restrictions.
 
- The generated wrappers are pre-generated and checked-in for Qt 4.7, 4.8, 5.0 and 5.3, so you only need to build and run the
+ The generated wrappers are pre-generated and checked-in for 5.0, 5.3, 5.4 and 5.6, so you only need to build and run the
  generator when you want to build additional wrappers or you want to upgrade/downgrade to another Qt version.
  You may use the generator to generate C++ bindings for your own C++ classes (e.g., to make them inheritable in Python),
  but this is currently not documented and involves creating your own typesystem files.
