@@ -234,6 +234,15 @@ QByteArray PythonQtMethodInfo::getInnerTemplateTypeName(const QByteArray& typeNa
   return QByteArray();
 }
 
+QByteArray PythonQtMethodInfo::getInnerListTypeName(const QByteArray& typeName)
+{
+  QByteArray name = getInnerTemplateTypeName(typeName);
+  if (name.isEmpty() && typeName.endsWith("List")) {
+    name = typeName.left(typeName.length() - 4);
+  }
+  return name;
+}
+
 int PythonQtMethodInfo::nameToType(const char* name)
 {
   if (_parameterTypeDict.isEmpty()) {
@@ -245,6 +254,10 @@ int PythonQtMethodInfo::nameToType(const char* name)
     _parameterTypeDict.insert("int", QMetaType::Int);
     _parameterTypeDict.insert("short", QMetaType::Short);
     _parameterTypeDict.insert("char", QMetaType::Char);
+    _parameterTypeDict.insert("signed long", QMetaType::Long);
+    _parameterTypeDict.insert("signed int", QMetaType::Int);
+    _parameterTypeDict.insert("signed short", QMetaType::Short);
+    _parameterTypeDict.insert("signed char", QMetaType::Char);
     _parameterTypeDict.insert("ulong", QMetaType::ULong);
     _parameterTypeDict.insert("unsigned long", QMetaType::ULong);
     _parameterTypeDict.insert("uint", QMetaType::UInt);
@@ -290,6 +303,21 @@ int PythonQtMethodInfo::nameToType(const char* name)
       _parameterTypeDict.insert("size_t", QMetaType::UInt);
     }
 
+#ifdef PYTHONQT_SUPPORT_ML_TYPES
+    _parameterTypeDict.insert("MLuint8",  QMetaType::UChar);
+    _parameterTypeDict.insert("MLint8",   QMetaType::Char);
+    _parameterTypeDict.insert("MLuint16", QMetaType::UShort);
+    _parameterTypeDict.insert("MLint16",  QMetaType::Short);
+    _parameterTypeDict.insert("MLuint32", QMetaType::UInt);
+    _parameterTypeDict.insert("MLint32",  QMetaType::Int);
+    _parameterTypeDict.insert("MLuint64", QMetaType::ULongLong);
+    _parameterTypeDict.insert("MLint64",  QMetaType::LongLong);
+    _parameterTypeDict.insert("MLuint",   QMetaType::ULongLong);
+    _parameterTypeDict.insert("MLint",    QMetaType::LongLong);
+    _parameterTypeDict.insert("MLfloat",  QMetaType::Float);
+    _parameterTypeDict.insert("MLdouble", QMetaType::Double);
+#endif
+
     // QVariant names
     _parameterTypeDict.insert("Q_LLONG", QMetaType::LongLong);
     _parameterTypeDict.insert("Q_ULLONG", QMetaType::ULongLong);
@@ -401,7 +429,11 @@ QString PythonQtSlotInfo::fullSignature(bool skipReturnValue, int optionalArgsIn
     if (sig.startsWith("new_")) {
       sig = sig.mid(4);
       isConstructor = true;
-    } else if (sig.startsWith("delete_")) {
+    }
+    else if (sig.startsWith("py_q_")) {
+      sig = sig.mid(5);
+    }
+    else if (sig.startsWith("delete_")) {
       sig = sig.mid(7);
       isDestructor = true;
     } else if(sig.startsWith("static_")) {
@@ -471,6 +503,9 @@ QByteArray PythonQtSlotInfo::slotName(bool removeDecorators) const
 {
   QByteArray name = PythonQtUtils::methodName(_meta);
   if (removeDecorators) {
+    if (name.startsWith("py_q_")) {
+      name = name.mid(5);
+    } else 
     if (name.startsWith("static_")) {
       name = name.mid(7);
       int idx = name.indexOf("_");
